import secrets

from .. import logger
from ..database import db
from ..models import ApiKey


def validate_api_key(api_key, permission):
    """Verify api key

    Verify a ApiKey so if the User has permission or not.
    Retrieves the access token if valid else retrieves False

    Args:
        api_key: ApiKey to verify
        permission: Permission needed to access restricted routes
    Returns:
        A ApiKey for this given Token
    Raises:
        Unauthorized: If api key is invalid
        Forbidden: If permission is insufficient
    """
    logger.debug("check api_key {{ {} }} is valid".format(api_key))
    api_key = ApiKey.query.filter_by(api_key=api_key).one_or_none()
    if api_key:
        logger.debug("api_key found")
        if not permission or api_key.user_.has_permission(permission):
            return api_key
        else:
            raise Forbidden
    logger.debug("no valid api key with api_key: {{ {} }} and permission: {{ {} }}".format(api_key, permission))
    raise Unauthorized


def create(user, name, description=None) -> ApiKey:
    """Create a ApiKey

    Args:
        user: For which User is to create a ApiKey

    Returns:
        A ApiKey for this given User
    """
    logger.debug("create api key token")
    token_str = secrets.token_hex(16)
    logger.debug("create api_key for user {{ {} }}".format(user))
    api_key = ApiKey(_user_id=user.id__, name=name, description=description, token=token_str)
    db.session.add(api_key)
    db.session.commit()
    return api_key


def get_users_api_keys(user) -> list[ApiKey]:
    """Get all ApiKeys for a User

    Args:
        user: For which User is to get all ApiKeys

    Returns:
        List of ApiKeys for this given User
    """
    return ApiKey.query.filter(ApiKey._user_id == user.id_).all()


def delete_api_key(api_key):
    """Delete a ApiKey

    Args:
        api_key: ApiKey to delete
    """
    logger.debug(f"delete api_key {{ {api_key} }} {{ {type(api_key)} }}")
    if isinstance(api_key, int):
        api_key = ApiKey.query.get(api_key)
    logger.debug("delete api_key {{ {} }}".format(api_key.token))
    db.session.delete(api_key)
    db.session.commit()
